
#ifndef MEM_H

#define MEM_H

/*===============================<o>=====================================

  This software was written by Ian L. Kaplan, Chief Fat Bear, Bear Products
  International.  Use of this software, for any purpose, is granted on two
  conditions: 

   1. This copyright notice must be included with the software
      or any software derived from it.

   2. The risk of using this software is accepted by the user.  No
      warranty as to its usefulness or functionality is provided
      or implied.  The author and Bear Products International provides
      no support.

 ===============================<o>=====================================*/

/*
  
  The mem_manage object supports memory allocation, deallocation and
  reuse.  It is built on top of the page_pool memory allocater, which
  uses large block allocation.  The memory allocation code uses
  a hash table to keep track of the size of allocated block.  The hash
  table support is provided by the hash object.

  Another way to keep track of the size of an allocated block is to store
  its size, along with the block itself.  The pointer to the allocated
  memory then points to the word ahead of the allocation size word.
  This has the potential disadvantage of being sensitive to application
  bugs that corrupt the block size.  By storing the block size elsewhere,
  the memory allocator may be more robust.

  A second alternative is to provide a "check sum" word, along with the
  block size.  When a block is freed, the "check sum" work is check to
  flag memory corruption.  In addition, a function can be implemented
  to walk through the allocated block chain, checking for corrupted
  memory block values.  In retrospect, I would probably use this
  algorithm, since the hash table has such high overhead (in both time
  and memory space).  What is really amazing is that even with all this
  overhead, this memory allocater is faster than Microsoft's "malloc" on
  Windows NT.

*/

#include "pagepool.h"
#include "hash.h"


class mem_manage : public page_pool, 
                   public hash {

private: // typedefs and variables
  typedef enum { min_block_size = 4 * sizeof(float) } mem_const;

protected:
  void *get_mem( unsigned int num_bytes )
  { return page_alloc( num_bytes ); }

public: // class functions
  void *mem_alloc( unsigned int num_bytes );
  void mem_free( void *addr );

}; // class mem_manage


#endif
